import _omit from 'lodash/omit';
import _pick from 'lodash/pick';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

/*
 * This file is part of the nivo project.
 *
 * Copyright 2016-present, Raphaël Benitte.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
import React from 'react';

import { TransitionMotion, spring } from 'react-motion';

import compose from 'recompose/compose';
import defaultProps from 'recompose/defaultProps';
import withPropsOnChange from 'recompose/withPropsOnChange';
import withStateHandlers from 'recompose/withStateHandlers';
import pure from 'recompose/pure';
import { pack } from 'd3-hierarchy';
import { withHierarchy, withTheme, withColors, withDimensions, withMotion } from '../../../hocs';
import { colorMotionSpring, getInterpolatedColor } from '../../../lib/colors';
import noop from '../../../lib/noop';
import { computeNodePath } from '../../../lib/hierarchy';
import Container from '../Container';
import { getAccessorFor } from '../../../lib/propertiesConverters';
import { bubblePropTypes, bubbleDefaultProps } from './props';

var ignoreProps = ['borderWidth', 'borderColor', 'enableLabel', 'label', 'labelFormat', 'labelTextColor', 'labelSkipRadius', 'labelTextDY', 'transitionDuration', 'transitionEasing'];

var nodeWillEnter = function nodeWillEnter(_ref) {
    var data = _ref.data;
    return _extends({
        scale: 0,
        r: 0,
        x: data.x,
        y: data.y
    }, colorMotionSpring(data.color));
};

var nodeWillLeave = function nodeWillLeave(springConfig) {
    return function (_ref2) {
        var data = _ref2.data;
        return _extends({
            scale: spring(0, springConfig),
            r: spring(0, springConfig),
            x: spring(data.x, springConfig),
            y: spring(data.y, springConfig)
        }, colorMotionSpring(data.color, springConfig));
    };
};

var computeZoom = function computeZoom(nodes, currentNodePath, width, height) {
    var currentNode = nodes.find(function (_ref3) {
        var path = _ref3.path;
        return path === currentNodePath;
    });
    if (currentNode) {
        var ratio = Math.min(width, height) / (currentNode.r * 2);
        var offsetX = width / 2 - currentNode.x * ratio;
        var offsetY = height / 2 - currentNode.y * ratio;
        nodes.forEach(function (node) {
            node.r = node.r * ratio;
            node.x = node.x * ratio + offsetX;
            node.y = node.y * ratio + offsetY;
        });
    }
};

var BubblePlaceholders = function BubblePlaceholders(_ref4) {
    var root = _ref4.root,
        getIdentity = _ref4.getIdentity,
        leavesOnly = _ref4.leavesOnly,
        namespace = _ref4.namespace,
        pack = _ref4.pack,
        width = _ref4.width,
        height = _ref4.height,
        margin = _ref4.margin,
        outerWidth = _ref4.outerWidth,
        outerHeight = _ref4.outerHeight,
        theme = _ref4.theme,
        getColor = _ref4.getColor,
        animate = _ref4.animate,
        motionStiffness = _ref4.motionStiffness,
        motionDamping = _ref4.motionDamping,
        isInteractive = _ref4.isInteractive,
        onClick = _ref4.onClick,
        children = _ref4.children,
        isZoomable = _ref4.isZoomable,
        zoomToNode = _ref4.zoomToNode,
        currentNodePath = _ref4.currentNodePath;

    // assign a unique id depending on node path to each node
    root.each(function (node) {
        node.id = getIdentity(node.data);
        node.path = computeNodePath(node, getIdentity);
    });

    pack(root);

    var nodes = leavesOnly ? root.leaves() : root.descendants();
    nodes = nodes.map(function (node) {
        node.color = getColor(_extends({}, node.data, { depth: node.depth }));

        return node;
    });

    if (currentNodePath) computeZoom(nodes, currentNodePath, width, height);

    var wrapperTag = void 0;
    var containerTag = void 0;

    var wrapperProps = {};
    var containerProps = {};

    if (namespace === 'svg') {
        wrapperTag = 'svg';
        containerTag = 'g';

        wrapperProps.width = outerWidth;
        wrapperProps.height = outerHeight;
        wrapperProps.xmlns = 'http://www.w3.org/2000/svg';
        containerProps.transform = 'translate(' + margin.left + ',' + margin.top + ')';
    } else {
        wrapperTag = 'div';
        containerTag = 'div';

        wrapperProps.style = {
            position: 'relative',
            width: outerWidth,
            height: outerHeight
        };
        containerProps.style = Object.assign({}, margin, {
            position: 'absolute'
        });
    }

    var nodeOnClick = noop;
    if (isInteractive) {
        if (isZoomable) {
            nodeOnClick = function nodeOnClick(node) {
                return function () {
                    onClick(node);
                    zoomToNode(node.path);
                };
            };
        } else {
            nodeOnClick = function nodeOnClick(node) {
                return function () {
                    return onClick(node);
                };
            };
        }
    }

    if (!animate) {
        return React.createElement(
            Container,
            { isInteractive: isInteractive, theme: theme },
            function (_ref5) {
                var showTooltip = _ref5.showTooltip,
                    hideTooltip = _ref5.hideTooltip;
                return React.createElement(wrapperTag, wrapperProps, React.createElement(containerTag, containerProps, children(nodes.map(function (node) {
                    return {
                        key: node.path,
                        data: node,
                        style: _pick(node, ['scale', 'r', 'x', 'y', 'color']),
                        onClick: nodeOnClick(node)
                    };
                }), { showTooltip: showTooltip, hideTooltip: hideTooltip, theme: theme })));
            }
        );
    }

    var springConfig = {
        stiffness: motionStiffness,
        damping: motionDamping
    };

    return React.createElement(
        Container,
        { isInteractive: isInteractive, theme: theme },
        function (_ref6) {
            var showTooltip = _ref6.showTooltip,
                hideTooltip = _ref6.hideTooltip;
            return React.createElement(wrapperTag, wrapperProps, React.createElement(
                TransitionMotion,
                {
                    willEnter: nodeWillEnter,
                    willLeave: nodeWillLeave(springConfig),
                    styles: nodes.map(function (node) {
                        return {
                            key: node.path,
                            data: node,
                            style: _extends({
                                scale: spring(1, springConfig),
                                r: spring(node.r, springConfig),
                                x: spring(node.x, springConfig),
                                y: spring(node.y, springConfig),
                                opacity: spring(1, springConfig)
                            }, colorMotionSpring(node.color, springConfig))
                        };
                    })
                },
                function (interpolatedStyles) {
                    return React.createElement(containerTag, containerProps, children(interpolatedStyles.map(function (interpolatedStyle) {
                        interpolatedStyle.style.color = getInterpolatedColor(interpolatedStyle.style);
                        interpolatedStyle.onClick = nodeOnClick(interpolatedStyle.data);

                        return interpolatedStyle;
                    }), { showTooltip: showTooltip, hideTooltip: hideTooltip, theme: theme }));
                }
            ));
        }
    );
};

BubblePlaceholders.propTypes = _omit(bubblePropTypes, ignoreProps);

var enhance = compose(defaultProps(_omit(bubbleDefaultProps, ignoreProps)), withHierarchy(), withDimensions(), withTheme(), withMotion(), withColors({ defaultColorBy: 'depth' }), withPropsOnChange(['identity'], function (_ref7) {
    var identity = _ref7.identity;
    return {
        getIdentity: getAccessorFor(identity)
    };
}), withPropsOnChange(['width', 'height', 'padding'], function (_ref8) {
    var width = _ref8.width,
        height = _ref8.height,
        padding = _ref8.padding;
    return {
        pack: pack().size([width, height]).padding(padding)
    };
}), withStateHandlers(function (_ref9) {
    var _ref9$currentNodePath = _ref9.currentNodePath,
        currentNodePath = _ref9$currentNodePath === undefined ? null : _ref9$currentNodePath;
    return {
        currentNodePath: currentNodePath
    };
}, {
    zoomToNode: function zoomToNode(_ref10) {
        var currentNodePath = _ref10.currentNodePath;
        return function (path) {
            if (path === currentNodePath) return { currentNodePath: null };
            return { currentNodePath: path };
        };
    }
}), pure);

export default enhance(BubblePlaceholders);